function [L,R]=RDS_generator(A,auto_range_flag,stereogram_type)
% RDS_generator     Random Dot Stereogram
%
% [L,R]=RDS_generator(A), where A is a grayscale mask image, returns the
% Random Dot Stereogram (RDS) of A.
% Mask A should be defined as a matrix containing shift values for each
% pixel. A value of 0 means the current pixel is not shifted. Any other
% integer value means the pixel is shifted by the same amount of pixels 
% (and thus should not be much larger than 20, assuming standard imge size). 
% In the stereogram, the regions of A with positive values
% will seem virtually fly in front of the screen, while those with
% negative values will seem to be behind the screen.
% 
% If you wish the function to automatically impose a range of values for
% shifting then auto_range_flag should be set to true (default).
% Otherwise, a suggested range of the values of A is [0,20]. For purposes
% of viewing the stereogram a range of [-15,15] is optimal.
% if auto_range_flag is set to false, the function will NOT autoscale the
% data, allowing you to set the absolute height of each level with respect
% to the screen.
%
% If no output is requested than the function will display the RDS for
% viewing:
% RDS_generator(A,auto_range, stereogram_type) specifies alternate viewing methods.
% The default is parallel-eyes method.  Available methods are:
%
%  'parallel-eyes'
%  'crossed-eyes'
%
% See http://www.vision3d.com/ to learn viewing stereograms.
%
% The vertical white line in the displayed stereogram helps the visualization.
% You can remove it to obtain a Single-Image Stereogram.
%
%
% Modified by Omri Perez, 2013, omriperez(at)gmail(dot)com
%
% Example:
% A=peaks(300);
% A=A-mean(mean(A));
% A=10*A/max(max(A));
% A=round(A);
% stereogram(A)
% [L,R] = stereogram(A)

if nargin<2 || auto_range_flag
    %make sure mask image is grayscale
    if size(A,3)>1
        A=rgb2gray(A);
    end
    A=im2double(A);
    %automatically set shift range
    A=(A).*20;
end

if nargin<3
    view_type = 'parallel-eyes'; % can be 'crossed-eyes'
else
    view_type = stereogram_type;
end



A=double(A);
A=round(A);

RDM = round(rand(size(A))); % Random Dot Matrix

% Initial left and right images
L = RDM;
R = RDM;

% For each region of A, a random dot pattern
% is shifted of an amount depending on the level
for k = min(min(A)):max(max(A))
    if k<0
        level = size(A,2)+k;
        TT = [RDM(:,level+1:end) RDM(:,1:level)];
    elseif k>0
        level = k;
        TT = [RDM(:,level+1:end) RDM(:,1:level)];
    elseif k==0
        level = 1;
        TT = RDM;
    end
    R(A==k) = TT(A==k);
end

if nargout==0 %if no output is requested then plot the RDS on screen
    % Selection of the view type
    % The 'ones(size(L,1),5)' is the white line
    if strcmp(view_type,'parallel-eyes')
        M = [L ones(size(L,1),5) R];
    elseif strcmp(view_type,'crossed-eyes')
        M = [R ones(size(L,1),5) L];
    end
    
    im_h = imagesc(M);
    axis off
    % truesize is necessary to avoid automatic scaling
    size_one2one(im_h)
    colormap gray
    
    set(gcf,'number','off','name','Stereogram - by I.-G. Marino')
    
    shg
end

function size_one2one(im_h)
ax_h = gca;
fig_h = gcf;

iw = size(get(im_h, 'CData'), 2);
ih = size(get(im_h, 'CData'), 1);

set(ax_h, 'Units', 'pixels');
set(fig_h, 'Units', 'pixels');
set(0, 'Units', 'pixels');

figurePos = get(fig_h, 'Position');

dap = get(0,'DefaultAxesPosition');

brdr_wi = round((1 - dap(3)) * iw / dap(3));
brdr_he = round((1 - dap(4)) * ih / dap(4));
brdr_le = floor(brdr_wi/2);
brdr_bo = floor(brdr_he/2);

RDS_fig_wi = iw + brdr_wi;
RDS_fig_he = ih + brdr_he;

figurePos(1) = figurePos(1) - floor((RDS_fig_wi - figurePos(3))/2);
figurePos(2) = figurePos(2) - floor((RDS_fig_he - figurePos(4))/2);
figurePos(3) = RDS_fig_wi;
figurePos(4) = RDS_fig_he;

axesPos(1) = brdr_le + 1;
axesPos(2) = brdr_bo + 1;
axesPos(3) = iw;
axesPos(4) = ih;

set(fig_h, 'Position', figurePos)
set(ax_h, 'Position', axesPos);

drawnow;
